#!/usr/local/bin/perl -w

#	Blogfeed 0.93
#	Blogfeed creates an RSS newsfeed from the HTML file 
#	of your blog. If you don't have the readme, you can
#	read the instructions at
#	http://kalsey.com/tools/blogfeed/
#	
#	The file readme.html should have been included 
#	with this script. Read it for instructions on how 
#	to us this.
#	
#	Blogfeed was created by
#	Adam Kalsey
#	adam@kalsey.com
#	http://www.kalsey.com/
#	
#	Visit http://kalsey.com/tools/blogfeed/ for 
#	the latest version.
#	
#	Permission is granted to use this script on your 
#	site in any way you please. If find it useful, 
#	please consider placing a link on your site to 
#	http://kalsey.com/tools/blogfeed/ so that 
#	others may use it too. Oh, and email me the URL of 
#	your blog so that I can check it out.

use strict;
$|++;
require "MD5.pm"; 
require "Simple.pm";
require "RSS.pm";

###################################################
## Configure this

my $blog_file = "index.html"; # You can also use a URL here
                   # like http://www.someserver.com/index.html
my $strip_html = "True"; # False if you want to keep HTML
my $ping = "True"; # False if you do not want to ping 
                   # Weblogs.com when your blog has changed

## End of configurable options
###################################################


# set some variables
my $channel_title;
my $channel_link;
my $channel_description;
my $count = 1;
my %in;
my $link;
my $stream;
my $line;


# Get the query string
if (length ($ENV{'QUERY_STRING'}) > 0){
    my $pair;
    my $name;
    my $value;
     my $buffer = $ENV{'QUERY_STRING'};
     my @pairs = split(/&/, $buffer);
     foreach $pair (@pairs){
          ($name, $value) = split(/=/, $pair);
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          $in{$name} = $value; 
     }
}


# Get the blog's HTML code
if ($blog_file =~ m{^http://})  {
 $stream = LWP::Simple::get($blog_file);
}
else {
 open(BLOG, "$blog_file") or die "Can't open blog: $!\n";
 undef $/;
 $stream = <BLOG>; 
 close BLOG;
}

# create RSS object
my $rss = XML::RSS->new( version => '0.92' );

print "content-type: text/xml\n\n";
# open the blog HTML file

my @html = split(/\n/, $stream);
foreach $line (@html){
   local $_ = $line;
   # grab the title from the HTML
    if (/<title>(.+?)<\/title>/) {
      $rss->channel(title => "$1");
      $rss->image(title => "$1");
    }
    
  # grab the description from the meta if it exists
    if (/<meta .*description.*\"(.+?)\"/) {
      $rss->channel(description => "$1");
    }

  # grab the link URL from the meta if it exists
    if (/<meta .*rss:link.*\"(.+?)\"/) {
      $channel_link = $1;
	  $channel_link =~ s/&/&amp;/g;
      $rss->channel(link => "$channel_link");
      $rss->image(link => "$channel_link");
    }
  # grab the language from the meta if it exists
    if (/<meta .*rss:language.*\"(.+?)\"/) {
      $rss->channel(language => "$1"); 
    }

  # grab the channel image URL from the meta if it exists
  # We'll use the channel's title and link for the image's
  #
    if (/<meta .*rss:image.*\"(.+?)\"/) {
      $rss->image(url => "$1");
    }
    
# find the span that contains the RSS description. From this we
# will create a synthetic title. We will also grab the URL of the
# first href and use that as the <link> value

		if (m#rss:item *\" *>(.+?)</span>#is){
			if ($count <= 15) {
				$count++;
				my $description = $1;
				my $title = $1;

    # Get the first link
				if ( /<\s*A\s+HREF\s*=\s*(["'])(.*?)\1.*?>/gi ) {
				   $link = $2;
				   $link =~ s/&/&amp;/g;
				}

    
    # the title is the first end of sentence punctuation,
    # if the title is longer than ten words, only use the first ten
    # words

	  	   	 	if ($title =~ m/(.+?)(?:<\/?br)/ig) { $title = $1;
				   $description =~ s/$title//ig; 
				}
				$title = &strip($title);
				$title = &encode($title);
				if ( $title =~ m#(.+?)[\.\?!]#is ) {
					$title = $1;
				}
				my $num_words=10;
				if (my ($words) = $title=~ /((?:\w+(?:\W+|$)){$num_words})/) {
					 $title = $words;
				}
				if ($strip_html) { $description = &strip($description); }
				$description = &encode($description);
				#$description =~ s/$title//ig;
                $rss->add_item(title => "$title",
                               description => "$description",
                               link => "$link");
			}
		}
}
&ping();


print $rss->as_string;

sub strip () {
	$_ = shift;
	s(<[^>]*>)( )g;
	return $_;
}

sub encode () {
	$_ = shift;
    s//\&iexcl;/g;
    s//\&cent;/g;
    s//\&pound;/g;
    s//\&curren;/g;
    s//\&yen;/g;
    s//\&brvbar;/g;
    s//\&sect;/g;
    s//\&uml;/g;
    s//\&copy;/g;
    s//\&ordf;/g;
    s/"/\&laquo;/g;
    s//\&reg;/g;
    s//\&macr;/g;
    s//\&deg;/g;
    s//\&plusmn;/g;
    s//\&sup2;/g;
    s//\&sup3;/g;
    s//\&acute;/g;
    s//\&micro;/g;
    s//\&middot;/g;
    s//\&cedil;/g;
    s//\&sup1;/g;
    s//\&ordm;/g;
    s//\&raquo;/g;
    s//\&frac14;/g;
    s//\&frac12;/g;
    s//\&frac34;/g;
    s//\&iquest;/g;
    s//\&Agrave;/g;
    s//\&Aacute;/g;
    s//\&circ;/g;
    s//\&Atilde;/g;
    s//\&Auml/g;
    s//\&ring;/g;
    s//\&AElig;/g;
    s//\&Ccedil;/g;
    s//\&Egrave;/g;
    s//\&Eacute;/g;
    s//\&Ecirc;/g;
    s//\&Euml;/g;
    s//\&Igrave;/g;
    s//\&Iacute;/g;
    s//\&Icirc;/g;
    s//\&Iuml;/g;
    s//\&ETH;/g;
    s//\&Ntilde;/g;
    s//\&Ograve;/g;
    s//\&Oacute;/g;
    s//\&Ocirc;/g;
    s//\&Otilde;/g;
    s//\&Ouml;/g;
    s//\&times;/g;
    s//\&Oslash;/g;
    s//\&Ugrave;/g;
    s//\&Uacute;/g;
    s//\&Ucirc;/g;
    s//\&Uuml;/g;
    s//\&Yacute;/g;
    s//\&THORN;/g;
    s//\&szlig;/g;
    s//\&agrave;/g;
    s//\&aacute;/g;
    s//\&acirc;/g;
    s//\&atilde;/g;
    s//\&auml;/g;
    s//\&aring;/g;
    s//\&aelig;/g;
    s//\&ccedil;/g;
    s//\&egrave;/g;
    s//\&eacute;/g;
    s//\&ecirc;/g;
    s//\&euml;/g;
    s//\&igrave;/g;
    s//\&iacute;/g;
    s//\&icirc;/g;
    s//\&iuml;/g;
    s//\&ieth;/g;
    s//\&ntilde;/g;
    s//\&ograve;/g;
    s//\&oacute;/g;
    s//\&ocirc;/g;
    s//\&otilde;/g;
    s//\&ouml;/g;
    s//\&divide;/g;
    s//\&oslash;/g;
    s//\&ugrave;/g;
    s//\&uacute;/g;
    s//\&ucirc;/g;
    s//\&uuml;/g;
    s//\&yacute;/g;
    s//\&thorn;/g;
    s//\&yuml;/g;
    s/"/\&quot;/g;
	s/&/\&amp;/g;
	s/</&amp;lt;/g;
	return $_;
}

sub ping () {
	my $thishash;
	my @lastping;
	my $response = " ";
    binmode(BLOG);
    $thishash = Digest::MD5->new->addfile(*BLOG)->hexdigest;
	my $now = time;
	open (CHECK, "+<Blogfeed.ping",) or die "Couldn't open the ping data file. $!\n";
	while(<CHECK>){
        chomp;
        @lastping = split('\|');
	}
  if ($thishash ne $lastping[0]) {
	if (($now - $lastping[1]) > 360) {
        my $ping = "name=$channel_title&url=$channel_link";
        $ping =~ s/ /+/g;
	print "\n<!-- Pinged Weblogs.com -->\n";
	$response = LWP::Simple::get("http://newhome.weblogs.com/pingSiteForm?$ping");
	if ($response ne '') {
		seek (CHECK,0,0);
	        truncate (CHECK,0);
	        print CHECK "$thishash|$now\n";
	}
	open (LOG, ">pinglog.html");
	print LOG "$response";
	close LOG;
	}
  }
	close CHECK;
}